##########################
# ORACLE DATABASE MODULE #
##########################

locals {
  # Naming & Tagging
  db_sid_name        = var.db_name == null ? format("db%s", substr(local.module_instance_id, 0, 6)) : var.db_name
  db_identifier_name = lower("${var.db_identifier_name}-${local.module_instance_id}")
  snapshot_name      = "${local.db_identifier_name}-final"
  db_owner_username  = "RDSMASTER"

  em_user_role = {
    "EMUSER.ABCC_EMEXPRESS_ROLE" = {
      "user" = "EMUSER"
      "role" = "ABCC_EMEXPRESS_ROLE"
    }
  }

  application_breakglass_user_role = {
    "APPLICATION_BREAKGLASS.ABCC_APPLICATION_BREAKGLASS" = {
      "user"           = "APPLICATION_BREAKGLASS"
      "role"           = "ABCC_APPLICATION_BREAKGLASS"
      "profile"        = "ABCC_TIME_LIMITED"
      "rotation_cycle" = "1"
      "rotation_duration" = null
    }
  }

  rw_user_role = {
    "ATA_RW.ABCC_DBAPPUSER" = {
      "user" = "ATA_RW"
      "role" = "ABCC_DBAPPUSER"
    }
  }

  ro_user_role = {
    "ATA_RO.ABCC_DBAPPUSER" = {
      "user" = "ATA_RO"
      "role" = "ABCC_DBAPPUSER"
    }
  }

  all_app_users_name  = setunion(["DBAPPUSER"], var.application_usernames)
  ApplicationUserRole = ["ABCC_DBAPPUSER"]
  app_users_role = {
    for app_user_role in setproduct(local.all_app_users_name, local.ApplicationUserRole) : "${app_user_role[0]}.${app_user_role[1]}" => {
      user = app_user_role[0]
      role = app_user_role[1]
    }
  }

  all_schema_users_name = setunion(["APPLICATION_SCHEMA"], var.schema_usernames)
  SchemaUserRole        = ["ABCC_APPLICATION_SCHEMA_ROLE , SQLT_USER_ROLE"]
  schema_users_role = {
    for schema_user_role in setproduct(local.all_schema_users_name, local.SchemaUserRole) : "${schema_user_role[0]}.${schema_user_role[1]}" => {
      user = schema_user_role[0]
      role = schema_user_role[1]
    }
  }

  goldengate_users_role = {
    "OGGADM.ABCC_OGGADM_ROLE" = {
      "user" = "OGGADM"
      "role" = "ABCC_OGGADM_ROLE"
      "profile" = "ADMIN"
      "rotation_cycle" = "2"
      "rotation_duration" = null
    }
  }

  bitooluserro_users_role = {
    "BITOOLUSERRO.ABCC_SHARED_INTERACTIVE_ROLE" = {
      "user" = "BITOOLUSERRO"
      "role" = "ABCC_SHARED_INTERACTIVE_ROLE"
      "profile" = "ABCC_SHARED_INTERACTIVE_RO"
      "rotation_cycle" = "1"
    }
  }

  hard_defaults = {
    engine                     = "oracle-ee"
    license_model              = "bring-your-own-license"
    port                       = 6140
    port_oem                   = 6141
    port_oem_agent             = 3872
    auto_minor_version_upgrade = false
    copy_tags_to_snapshot      = true
    storage_encrypted          = true
  }


  db_users_and_roles = merge(local.em_user_role, local.app_users_role, local.schema_users_role, local.application_breakglass_user_role, local.rw_user_role, local.ro_user_role)
  db_users_and_roles_1 = var.goldengate_replication.enable_goldengate_replication == true ? merge(local.db_users_and_roles, local.goldengate_users_role) : local.db_users_and_roles
  db_users_and_roles_2 = var.add_bitooluserro == true ? merge(local.db_users_and_roles_1, local.bitooluserro_users_role) : local.db_users_and_roles_1

  # db_users_and_roles = var.goldengate_replication.enable_goldengate_replication == true ? merge(
  # local.em_user_role, local.app_users_role, local.schema_users_role, local.application_breakglass_user_role, local.rw_user_role, local.ro_user_role, local.goldengate_users_role) : merge(
  # local.em_user_role, local.app_users_role, local.schema_users_role, local.application_breakglass_user_role, local.rw_user_role, local.ro_user_role)

  # Networking
  default_vpc_label = "Spoke VPC"

  # define previous and new major engine version and parameter group family
  db_major_engine_version   = var.upgrade_from_12c ? "12.1" : "19"
  db_major_engine_version19   = "19" 
  db_parameter_group_family = var.upgrade_from_12c ? "oracle-ee-12.1" : "oracle-ee-19"
  db_parameter_group_family19 = "oracle-ee-19"
  app_account_environment = trim( upper(data.aws_ssm_parameter.account_environment.value), " " )

  # skip_final_snapshot and deletion_protection
  skip_final = local.app_account_environment == "PROD" && var.deletion_protection == false ? false : var.skip_final_snapshot

  immutable_backups = local.app_account_environment == "PROD" ? true : var.enable_immutable_backups
  backup_vault_retention = local.app_account_environment == "PROD" ? var.backup_vault_retention_period : ( var.enable_immutable_backups == false && local.app_account_environment != "PROD" ? "NOBACKUP" : var.backup_vault_retention_period )
  pitr_snap_backup_tag = var.backup_vault_retention_snap_period == "NOBACKUP" ? "PITRBACKUP" : "${var.backup_vault_retention_snap_period}DAYS"

  snapshot_name_for_db_clone = var.snapshot_id_for_clone
  # if both snapshot_id_for_clone and restore_point_in_time are specified only snapshot_id_for_clone will be honored
  restore_2_pit              = var.snapshot_id_for_clone != null? null : var.point_in_time_for_clone.restore_time == null && var.point_in_time_for_clone.source_db_instance_identifier == null ? null : [
    var.point_in_time_for_clone
  ]
  pit_db_instance_identifier = local.restore_2_pit != null ? var.point_in_time_for_clone.source_db_instance_identifier : null
  database_clone             = local.snapshot_name_for_db_clone != null || local.restore_2_pit != null ? true : false
  major_version_str = "'${var.major_version}'"
  version_map  = split(",", join(",", [for j in [for i in tolist(split("],", data.aws_ssm_parameter.version_map.value)): trim(i, "[ ]") if can(regex(local.major_version_str, i))]: j]))
  db_version = trim(([for i in local.version_map: trim(local.version_map[index(local.version_map, i) + 1], "( )") if can(regex(local.app_account_environment, i))][0]), "' '")

  backup_rule        = local.backup_vault_retention
  re_name            = substr(upper(var.db_identifier_name), 0, 30)
  start_window       = "60"
  completion_window  = "720"

  db_backup_start = split("-", var.db_backup_window)[0]
  db_backup_hour_start_orig = tonumber(split(":", local.db_backup_start)[0])
  db_backup_minute_start = split(":", local.db_backup_start)[1]

  #db_backup_hour_start = (local.db_backup_hour_start_orig <= 21) ? local.db_backup_hour_start_orig : 21
  #cron_backup_hours = (local.db_backup_hour_start_orig <= 21) ? local.db_backup_hour_start_orig + 1 : 22
  cron_backup_hours_str = format("%02d", local.db_backup_hour_start_orig)
  cron_backup_mins_str = local.db_backup_minute_start

  ## User passed db_backup_window is used to determine start
  ## This variable is used only on first run. After that, AWS Backup takes responsibility of maintianing backups
  ## Continuous backup is enabled and Automatated backup is disabled at DB level.
  db_backup_window_static = "21:00-22:00"

  #timestamps         = join(":", [var.immutable_backup_hour, var.immutable_backup_minute]) 
  timestamps         = local.db_backup_start
  backup_plan_name   = join("-", ["DP", "ORA",local.re_name, local.timestamps, "${local.backup_rule}DAYS"])
  backup_plan_name_snap   = join("-", ["DP", "ORA",substr(upper(local.re_name),0,15), replace(replace(replace(replace(replace(replace(replace(var.snapshot_cron," ","-"),",","-"),"*","."),"?","."),"cron",""),"(",""),")",""),"${var.backup_vault_retention_snap_period}DAYS"])
  bck_selection_name = join("-", [local.re_name, "backup-selection"])
  other_mandatory_tags = {
    BACKUP = "CENTRALIZED"
  }
}


############################
# Create Random Password   #
############################
resource "random_string" "db_owner_initial_password" {
  length           = 30
  override_special = "_!%^"
  special          = true
  min_special      = 1
}

resource "random_password" "uniq_str" {
  length           = 10
  special          = false
}

######################################################################################
# Retrieve Security Group id from shared_rds_oracle_config-lambda-[version] in order   #
# to attach it to RDS VPC security groups so the lambda has access to update the DB. #
######################################################################################

data "aws_cloudformation_export" "rds_oracle_config_lambda_security_group_id" {
  name = "apollo-RdsOracleConfigLambdaSecurityGroup"
}

resource "aws_security_group" "rds_oracle_customer_managed_security_group" {
  name        = "rds-oracle-customer-managed-security-group-${local.module_instance_id}"
  description = "Empty Oracle security group for users to use to attach security group(s) from other services which needs to communicate with RDS cluster."
  vpc_id      = data.aws_vpc.vpc.id
  tags        = local.tags
}

resource "aws_security_group" "database" {
  name        = "${local.db_identifier_name}-RDSOracleCustomSecurityGroup"
  vpc_id      = data.aws_vpc.vpc.id
  description = "RDS Oracle database Security Group."
  tags        = local.tags

  lifecycle {
    create_before_destroy = true
  }
}

resource "aws_security_group_rule" "database_egress_rule" {
  type        = "egress"
  from_port   = 0
  to_port     = 0
  protocol    = -1
  cidr_blocks = ["0.0.0.0/0"]

  security_group_id = aws_security_group.database.id
}

resource "aws_security_group_rule" "rds_oracle_config_lambda_security_group_ingress_rule" {
  description              = "shared-RdsOracleConfig-Lambda security group used for configuring Oracle DB."
  type                     = "ingress"
  from_port                = lookup(local.hard_defaults, "port")
  to_port                  = lookup(local.hard_defaults, "port_oem")
  protocol                 = "tcp"
  source_security_group_id = data.aws_cloudformation_export.rds_oracle_config_lambda_security_group_id.value
  security_group_id        = aws_security_group.database.id
}

################################################
# Create IAM monitoring role and attach policy #
################################################
data "aws_iam_policy_document" "oracle_monitoring_assume_role_document" {
  statement {
    effect = "Allow"
    principals {
      type        = "Service"
      identifiers = ["monitoring.rds.amazonaws.com"]
    }
    actions = ["sts:AssumeRole"]
  }
}

resource "aws_iam_role" "oracle_monitoring_role" {
  name               = "${local.db_identifier_name}-monitoring_role"
  assume_role_policy = data.aws_iam_policy_document.oracle_monitoring_assume_role_document.json
  tags               = local.tags
}

resource "aws_iam_role_policy_attachment" "oracle_monitoring_attachment" {
  role       = aws_iam_role.oracle_monitoring_role.name
  policy_arn = "arn:aws:iam::aws:policy/service-role/AmazonRDSEnhancedMonitoringRole"
}

#########################################################
# Create Subnet Group, Parameter Group and Option Group #
#########################################################
module "rds-oracle-subnet-groups" {
  source                      = "./modules/rds-oracle-subnet-groups"
  module_instance_id          = local.module_instance_id
  targeted_private_subnet_ids = var.targeted_private_subnet_ids
  tags                        = local.instance_tags
  db_identifier_name          = local.db_identifier_name
  use_private_subnets         = var.use_private_subnets
}

module "rds-oracle-parameter-option-groups" {
  source                 = "./modules/rds-oracle-parameter-option-groups"
  module_instance_id     = local.module_instance_id
  parameter_group_family = local.db_parameter_group_family
  parameter_group_family19 = local.db_parameter_group_family19
  custom_db_parameters   = var.custom_db_parameters
  parameter_fix_control_value = var.parameter_fix_control_value
  engine_name            = lookup(local.hard_defaults, "engine")
  major_engine_version   = local.db_major_engine_version
  major_engine_version19 = local.db_major_engine_version19
  port_oem               = lookup(local.hard_defaults, "port_oem")
  security_group_id      = [aws_security_group.database.id]
  tags                   = local.instance_tags
  goldengate_replication = var.goldengate_replication
  custom_sga             = var.custom_sga
  custom_pga             = var.custom_pga
  db_16k_cache_size      = var.db_16k_cache_size
  db_32k_cache_size      = var.db_32k_cache_size
  db_identifier_name     = local.db_identifier_name
  upgrade_from_12c       = var.upgrade_from_12c
  timezone               = var.timezone
  cross_region_readreplica_flg = var.cross_region_readreplica_flg
  replicate_source_db    = var.replicate_source_db
  oem_agent              = var.oem_agent
  oem_agent_rgs_key      = var.oem_agent_rgs_key
}


###################
# Create Database #
###################
resource "aws_db_instance" "rds" {
  identifier     = local.db_identifier_name
  engine         = var.cross_region_readreplica_flg != true ? lookup(local.hard_defaults, "engine") : null
  engine_version = var.cross_region_readreplica_flg != true && local.database_clone != true ? local.db_version : null
  instance_class = var.instance_class

  # Database Storage
  storage_type      = var.storage_type
  allocated_storage = var.cross_region_readreplica_flg !=  true ? var.allocated_storage : null
  max_allocated_storage = var.max_allocated_storage
  #iops              = var.storage_type == "io1" ? var.db_iops : 0
  iops                 = var.storage_type == "io1" || var.storage_type == "gp3" && var.allocated_storage > 200 ? var.db_iops : null
  storage_throughput    = var.storage_type == "gp3" && var.allocated_storage > 200 ? var.db_throughput : null

  # Encryption
  storage_encrypted = lookup(local.hard_defaults, "storage_encrypted")
  kms_key_id        = var.db_kms_key_arn
  license_model     = lookup(local.hard_defaults, "license_model")
  ca_cert_identifier  = var.ca_cert_identifier

  # Connection Details
  db_name  = var.cross_region_readreplica_flg !=  true ? upper(local.db_sid_name) : null
  username = var.cross_region_readreplica_flg !=  true ? local.db_owner_username : null
  password = (var.cross_region_readreplica_flg !=  true ? random_string.db_owner_initial_password.result : "")
  port     = lookup(local.hard_defaults, "port")

  # Make the db_name upper-case.  Oracle sets table names, user names, and database names to upper-case.

  parameter_group_name = var.upgrade_from_12c ? (var.custom_parameter_group == "NONE" ? module.rds-oracle-parameter-option-groups.parameter_group_name19 : var.custom_parameter_group) : (var.custom_parameter_group == "NONE" ? module.rds-oracle-parameter-option-groups.parameter_group_name : var.custom_parameter_group)

  option_group_name    = var.upgrade_from_12c ? (var.cross_region_readreplica_flg != true ? module.rds-oracle-parameter-option-groups.options_group_name19 : null) : (var.cross_region_readreplica_flg != true ? module.rds-oracle-parameter-option-groups.options_group_name : null)

  tags                 = local.instance_tags

  replicate_source_db       = (var.cross_region_readreplica_flg ==  true ? var.replicate_source_db : null)
  replica_mode              = (var.cross_region_readreplica_flg ==  true ? var.replica_mode : null)
  backup_window             = local.backup_vault_retention == "NOBACKUP" ? var.db_backup_window : null

  backup_retention_period   = local.backup_vault_retention == "NOBACKUP" ? var.db_backup_retention_period : null
  deletion_protection       = var.deletion_protection
  delete_automated_backups  = local.backup_vault_retention == "NOBACKUP" ? true : var.delete_automated_backups
  skip_final_snapshot       = (var.cross_region_readreplica_flg !=  true) ? local.skip_final : true
  final_snapshot_identifier = local.snapshot_name

  copy_tags_to_snapshot     = lookup(local.hard_defaults, "copy_tags_to_snapshot")

  # Network / Security
  vpc_security_group_ids = flatten([
    aws_security_group.database.id,
    aws_security_group.rds_oracle_customer_managed_security_group.id
  ])
  db_subnet_group_name = module.rds-oracle-subnet-groups.name
  multi_az             = var.multi_az

  performance_insights_enabled          = var.performance_insights_enabled
  performance_insights_kms_key_id       = (var.performance_insights_enabled == true ? var.db_kms_key_arn : "")
  performance_insights_retention_period = (var.performance_insights_enabled == true ? var.performance_insights_retention_period : null)


  apply_immediately           = var.db_apply_immediately
  maintenance_window          = var.db_maintenance_window
  allow_major_version_upgrade = true
  auto_minor_version_upgrade = false

  # Monitoring
  monitoring_role_arn             = var.db_monitoring_interval != 0 ? aws_iam_role.oracle_monitoring_role.arn : ""
  monitoring_interval             = var.db_monitoring_interval
  enabled_cloudwatch_logs_exports = ["alert", "audit", "listener", "trace"]

  # Data
  character_set_name = var.character_set_name

  depends_on = [
    module.rds-oracle-parameter-option-groups.parameter_group_name,
    module.rds-oracle-parameter-option-groups.options_group_name,
    module.rds-oracle-subnet-groups.name
  ]

  dynamic "timeouts" {
    for_each = var.timeouts
    content {
      create = timeouts.value["create"]
      delete = timeouts.value["delete"]
    }
  }

  snapshot_identifier = local.snapshot_name_for_db_clone

  lifecycle {
    ignore_changes = [db_name, snapshot_identifier, engine_version, tags["PATCH_DATABASE_UPGRADE"]]
  }
}

# Adding delay on destroy to handle race condition
resource "time_sleep" "wait_5mins_remove_s3_role" {
  destroy_duration = "5m"
}

######################################
# Create IAM Role for S3 Integration #
######################################
resource "aws_iam_role" "s3_integration" {
  name               = var.cross_region_readreplica_flg ? "RdsS3IntegrationRole-${local.db_identifier_name}-crossregion" : "RdsS3IntegrationRole-${local.db_identifier_name}"
  path               = "/s3/"
  assume_role_policy = data.aws_iam_policy_document.oracle_rds_assume_role_policy.json
  tags               = local.tags
}

data "aws_iam_policy_document" "oracle_rds_assume_role_policy" {
  statement {
    effect = "Allow"
    principals {
      type        = "Service"
      identifiers = ["rds.amazonaws.com"]
    }
    actions = ["sts:AssumeRole"]
  }
}

resource "aws_db_instance_role_association" "oracle_rds_s3_integration" {
  db_instance_identifier = aws_db_instance.rds.id
  feature_name           = "S3_INTEGRATION"
  role_arn               = aws_iam_role.s3_integration.arn
  depends_on = [
    time_sleep.wait_5mins_remove_s3_role
  ]
}
