####################
# DATABASE DETAILS #
####################

#variable "engine_version" {
#  description = "Engine version of Oracle"
#  type        = string
#  default     = "19.0.0.0.ru-2022-01.rur-2022-01.r1"
#
#  validation {
#    condition = (
#    can(regex("19.0.0.0", var.engine_version))
#    )
#    error_message = "The engine_version value should match RDS Oracle 19c versions  i.e. \"19.0.0.0\". Note that the version names provided should match what you see here: https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.Oracle.PatchComposition.html for example 19.0.0.0.ru-2022-01.rur-2022-01.r1."
#  }
#}

variable "major_version" {
  description = "Enter two digit major engine version number i.e. '19' for Oracle RDS major engine version."
  type        = string
  default     = "19"
  validation {
    condition = (
        can(regex("^[1-2][0-9]$", var.major_version))
        )
    error_message = "Please enter valid two digit Oracle Database 'major' engine version number example 19."
  }
}

variable "upgrade_from_12c" {
  description = "Specifies if the database was originally provisioned as RDS Oracle version 12c and is being upgraded to version 19c. If set to \"true\", this variable should remain \"true\" until this variable is removed"
  type        = bool
  default     = false
}

variable "instance_class" {
  description = "The instance type of the RDS instance"
  type        = string
  default     = "db.m5.large"
}

//TODO: do we need 4 subnet variables here? Opportunity to fix in RDS Oracle 5
variable "targeted_private_subnet_ids" {
  type        = set(string)
  default     = null
  description = <<EOF
If you need to target specific Subnets, then provide a minimum of 2 private
subnets ids in the following format in order to support multiple availability zones.
We will validate and remove any deemed public.
Example: `["subnet-XXXXXXX1", "subnet-XXXXXXX1"]`
EOF
}

# Database Storage
variable "storage_type" {
  description = "Type of storage gp2, gp3 and io1,The default is 'gp2'"
  type        = string
  default     = "gp2"
}

variable "allocated_storage" {
  description = "The allocated storage in gigabytes"
  type        = number
  default     = 100
}

variable "max_allocated_storage" {
  description = "The upper limit to which RDS can automatically scale the storage in GBs, Must be greater than or equal to allocated_storage or 0 to disable Storage Autoscaling."
  type        = number
  default     = 0
}

# Connection Details

variable "skip_final_snapshot" {
  description = "Determines whether a final DB snapshot is created before the DB instance is deleted. If true is specified, no DBSnapshot is created. If false is specified, a DB snapshot is created before the DB instance is deleted, using the value from final_snapshot_identifier"
  type        = bool
  default     = true
}

variable "deletion_protection" {
  description = "Determines whether delete protection is set"
  type        = bool
  default     = true
}

variable "delete_automated_backups" {
  description = "Determines whether delete protection is set"
  type        = bool
  default     = false
}

variable "multi_az" {
  description = "Specifies if the RDS instance is multi-AZ"
  type        = bool
  default     = false
}

variable "performance_insights_enabled" {
  description = "(Optional) Specifies whether Performance Insights are enabled. Defaults to false."
  type        = bool
  default     = false
}

variable "performance_insights_retention_period" {
  description = <<DESC
  (Optional) The amount of time in days to retain Performance Insights data. Either 7 (7 days) or 731 (2 years).
When specifying performance_insights_retention_period, performance_insights_enabled needs to be set to true.
Defaults to '7'.
DESC
  type        = number
  default     = 7
  validation {
    condition = (
    contains([7, 731], var.performance_insights_retention_period)
    )
    error_message = "The retention value must be either 7 or 731."
  }
}

variable "character_set_name" {
  description = "The character set name to use for DB encoding in Oracle instances. This can't be changed. See Oracle Character Sets Supported in Amazon RDS for more information"
  type        = string
  default     = "AL32UTF8"
}

variable "timezone" {
  description = "(Optional) The TIMEZONE for Database option group. Amazon defaults DB Timezone to 'UTC' if not defined. The time zone option is a permanent and persistent option; Once defined, it cannot be changed. For more details and supported list please refer here : https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.Oracle.Options.Timezone.html ."
  type        = string
  default     = null
  validation {
    condition     = (
    (var.timezone != null ? length(var.timezone) >= 3 : true) &&
    (var.timezone != null ? can(regex("UTC|Etc|Africa|America|Asia|Atlantic|Australia|Brazil|Canada|Europe|Pacific|US", var.timezone)) : true)
    )
    error_message = "Timezone provided must be in the list supported by Amazon here: https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.Oracle.Options.Timezone.html#Appendix.Oracle.Options.Timezone.Zones . Examples include UTC, America/New_York, US/Central ."
  }
}

variable "rotation_cycle" {
  description = "Rotation cycle of the secret e.g. every 1 day"
  type        = string
  default     = "1"

  validation {
    condition = (
    contains(["1", "30", "90"], var.rotation_cycle)
    )
    error_message = "The rotation_cycle value must be either \"1\" or \"30\"  or \"90\"."
  }
}

variable "rotation_duration" {
  description = "Rotation window duration - the secret rotation window always starts at midnight (UTC), the duration (in hours) allows you to control when it ends e.g. 1hr would give a rotation window starting at 23:59:59 until 00:59:59. The default is null which means the duration window is 24 hours (full day)."
  type        = string
  default     = null
  validation {
    condition = (
      var.rotation_duration != null ? contains(["1h","2h","3h","4h","5h","6h","7h","8h","9h","10h","11h","12h","13h","14h","15h","16h","17h","18h","19h","20h","21h","22h","23h","24h"], var.rotation_duration) : true
    )
    error_message = "Duration must be null or between 1h and 24h."
  }
}

variable "tags" {
  description = "A list of tag blocks"
  type        = map(string)
  default     = {}
}

variable "db_iops" {
  description = "The amount of provisioned IOPS. gp3 baseline IOPS is 3000 for less than 200 GiB Storage and set 12000-64000 IOPS greater than 200GiB Storage "
  type        = number
  default     = 3000
}

# Encryption
variable "db_kms_key_arn" {
  description = "The ARN for the KMS encryption key. If creating an encrypted replica, set this to the destination KMS ARN."
  type        = string
}

variable "db_throughput" {
  description = "The throughput volume supports for gp3, in MiB/s. Baseline throughput 125 MiB/s for less then 200 GiB Storage and greater then 200 GiB Storage throughput 500-4000 MiB/s "
  type        = number
  default     = 125
}

# Backups / Snapshots
variable "db_backup_window" {
  description = "The daily time range (in UTC) during which automated backups are created if they are enabled. Must not overlap with maintenance_window"
  type        = string
  default     = "21:00-22:00"
}

variable "db_backup_retention_period" {
  description = "The days to retain backups for. Must be between 0 and 35. Must be greater than 0 if the database is used as a source for a Read Replica"
  type        = number
  default     = 2

  validation {
    condition = (
    var.db_backup_retention_period >= 1 &&
    var.db_backup_retention_period <= 35
    )
    error_message = "Must be between 1 and 35, inclusive. See AWS ES service limits for further details."
  }
}

variable "backup_vault_retention_period" {
  description  = "specify valid retention for backup vault"
  type = string
  default     = "35"
  validation {
    condition     = contains(["35","NOBACKUP"], var.backup_vault_retention_period)
    error_message = "Please choose the value specified in the variable rules."
  }
}

variable "backup_vault_retention_snap_period" {
  description  = "specify valid retention for snapshot backup"
  type = string
  default     = "NOBACKUP"
  validation {
    condition     = contains(["35","70","100","400","NOBACKUP"], var.backup_vault_retention_snap_period)
    error_message = "Please choose the value specified in the variable rules."
  }
}

# Updates / Maintenance
variable "db_apply_immediately" {
  description = "Specifies whether any database modifications are applied immediately, or during the next maintenance window."
  type        = bool
  default     = true
}

variable "db_maintenance_window" {
  description = "The window to perform maintenance in. Syntax: ddd:hh24:mi-ddd:hh24:mi. (in UTC)"
  type        = string
  default     = "Sun:22:00-Sun:23:00"
}

# Monitoring
variable "db_monitoring_interval" {
  description = "The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance. To disable collecting Enhanced Monitoring metrics, specify 0"
  type        = number
  default     = 60
}

#####################
# Replica Variables #
#####################
variable "create_read_replica" {
  description = "Determines whether to create a replica DB or not. If set to true, enable read replica"
  type        = bool
  default     = false
}

variable "replica_mode" {
  description = "(Optional) the replica mode to use for the database.  Default is 'open-read-only'"
  type        = string
  default     = "open-read-only"
  validation {
    condition     = contains(["open-read-only", "mounted"], var.replica_mode)
    error_message = "Replica_mode must be one of: 'open-read-only, mounted'."
  }
}

variable "application_usernames" {
  type        = set(string)
  description = "Custom application user names to add to database. Example: `[ \"AppUser1\", \"AppUser2\" ]`"
  default     = []
}

variable "schema_usernames" {
  type        = set(string)
  description = "Custom schema user names to add to database.  Example: `[ \"SchemaUser1\", \"SchemaUser2\" ]`"
  default     = []
}

variable "add_bitooluserro" {
  description = "Add the BITOOLUSERRO SHARED interactive user"
  type        = bool
  default     = false
}

#################################
# Shared Service Catalog Lmabda #
#################################

#variable "shared_lambda_rds_master_secrets_rotation_function_version" {
#  description = "Version of the rds admin password lambda"
#  type        = string
#  default     = ""
#}

#variable "shared_lambda_rds_user_secrets_rotation_func_version" {
#  description = "Version of the rds user password lambda"
#  type        = string
#  default     = "2-5-5"
#
#  validation {
#    condition = (
#    can(regex("^(\\d)-(\\d)-(\\d)", var.shared_lambda_rds_user_secrets_rotation_func_version))
#    )
#    error_message = "The rds user password version value must start with with (\\d)-(\\d)-(\\d)."
#  }
#}

variable "custom_sga" {
  description = "(Optional) The default value is 3/4"
  type        = number
  default     = null
}

variable "custom_pga" {
  description = "(Optional) The default value is 1/8"
  type        = number
  default     = null
}

variable "db_16k_cache_size" {
  description = "db_16k_cache_size in bytes The default value is 0"
  type        = number
  default     = null
}

variable "db_32k_cache_size" {
  description = "db_32k_cache_size in bytes The default value is 0"
  type        = number
  default     = null
}

########################
# GoldenGate Variables #
########################

variable "goldengate_replication" {
  description = "Enables the Golden Gate Replication parameter. This must be enabled on all GoldenGate source databases."
  type = object({
    enable_goldengate_replication = bool
    engine_version                = string
  })

  default = {
    enable_goldengate_replication = false
    engine_version                = "19.0.0.0.ru-2022-01.rur-2022-01.r1"
  }

  validation {
    condition = (
    var.goldengate_replication.enable_goldengate_replication == false ||
    ! can(regex("12.2.0.1|12.1.0.2|19.0.0.0.ru-2020-01", var.goldengate_replication.engine_version))
    )
    error_message = "GoldenGate is only available on Oracle Database version 19c198 and above."
  }
}


variable "db_name" {
  description = "Oracle database SID name"
  type        = string
  default     = null

  validation {
    condition     = var.db_name != null ? length(var.db_name) < 9 : true
    error_message = "The Oracle Database SID name can only be 8 characters long."
  }

}

variable "db_identifier_name" {
  description = "(Required) Oracle database identifier name"
  type        = string

  validation {
    condition     = length(var.db_identifier_name) < 40
    error_message = "The Oracle Database identifier name must less than 40 characters long since we append '-replica' to read-replica identifier name and generated module instance id which come extremely close to the 63 characters limit."
  }
}

variable "use_private_subnets" {
  type        =  bool
  description = "Whether to use private subnets or not"
  default     = true
}

variable "subnets" {
  type        = list(string)
  description = "List of Subnet Names as passed in by the user.  Good practice is to return any variables that are passed in by the user"
  default     = []
}

variable "enable_immutable_backups" {
  description = "Determines whether immutable backups are enabled. this variable is overridded to be true for Gold-Prod databases. In lower environments users can opt to enable immutable backups for testing"
  type        = bool
  default     = false
}

variable "immutable_backup_hour" {
    description = "Immutable backup schedule is in the form of a Cron expression constructed using variables immutable_backup_hour and immutable_backup_minute"
  type    = string
  default   = "23"
}
# Customer to choose this. Used with variable 'immutable_backup_hour'. Should not conflict with 'Automated backup' schedule.  Default is 0 for top of the hour 23:00 (11:00 PM UTC)
variable "immutable_backup_minute" {
  description = "Immutable backup schedule is in the form of a Cron expression constructed using variables immutable_backup_hour and immutable_backup_minute"
  type  = string
  default  = "0"
}

variable "recovery_window_in_days" {
  description = "(Optional) The default value is 0 (As per Application Team requirement .Number of days that AWS Secrets Manager Password waits before it can delete a secret. This value can be 0 to force deletion without recovery or range from 7 to 30 days)"
  type        = number
  default     = 0
}

variable "snapshot_id_for_clone" {
  description = "To create a new database provide a snapshot identifier. This variable when specified will override point_in_time_for_clone variable"
  type = string
  default = null
}


variable "point_in_time_for_clone" {
  description = "(Optional) To create a new database specify a restore time and source database identifier.   Restore time can be one of \"latest\" or UTC timestamp. point_in_time_for_clone will be overridden if snapshot_id_for_clone variable is also specified."
  type        = object(
    {
      restore_time                  = string
      source_db_instance_identifier = string
    }
  )
  default = {
    restore_time                  = null
    source_db_instance_identifier = null
  }
  validation {
    condition     = var.point_in_time_for_clone.restore_time == null || var.point_in_time_for_clone == "latest" || can(regex("^[0-9-]+T[0-9:]+Z|latest", var.point_in_time_for_clone.restore_time))
    error_message = "Invalid restore_time. Must be either \"latest\" or in UTC format like example 2015-03-07T23:45:00Z."
  }
}

variable "custom_db_parameters" {
  description = "List of custom db parameters"
  type        = list(map(string))
  default     = []

  validation {
    condition = alltrue([
    for v in var.custom_db_parameters : contains([
      "java_jit_enabled",
      "open_cursors",
      "optimizer_adaptive_plans",
      "optimizer_adaptive_statistics",
      "optimizer_features_enable",
      "optimizer_index_cost_adj",
      "optimizer_use_invisible_indexes",
      "parallel_degree_policy",
      "resource_manage_goldengate",
      "skip_unusable_indexes",
      "resource_manager_plan",
      "max_string_size",
      "_bug29394014_allow_triggers_on_vpd_table"
    ], v.name)
    ])
    error_message = "Valid modifiable DB parameters are: (java_jit_enabled, open_cursors, optimizer_adaptive_plans, optimizer_adaptive_statistics, optimizer_features_enable, optimizer_index_cost_adj, optimizer_use_invisible_indexes, parallel_degree_policy, resource_manage_goldengate, skip_unusable_indexes, resource_manager_plan, max_string_size, _bug29394014_allow_triggers_on_vpd_table)."
  }
}

variable "parameter_fix_control_value" {
  description = "Parameters to apply _fix_control bug fix"
  type = string
  default     = null
  validation {
    condition     = (
    (var.parameter_fix_control_value != null ? can(regex("^(([0-9]{8}[:][01])+(\\,)?)*$[^#$%^&*()<>'a-zA-A]*$", var.parameter_fix_control_value)) : true) &&
    (var.parameter_fix_control_value != null ? can(regex(".*[^,]$", var.parameter_fix_control_value)) : true)
    )
    error_message = "Parameter fix control enable and disable value should be in number 31459242:1 or 31459242:0 ."
  }
}

variable "cross_region_readreplica_flg" {
  description = "(Optional) cross_region_readreplica_flg flag will be enabled ONLY when Secondary Database needs to be in cross region"
  type        = bool
  default     = false
}

variable "replicate_source_db" {
  description = "(Optional) replicate_source_db will be set to Source Database identifier when cross region replicaiton is enabled"
  type        = string
  default     = null
}

variable "replica_secret_region" {
  description = "SecretManager to be replicated from Primary to this Secondary region"
  type        = string
  default     = null
  validation {
    condition     = (
      (var.replica_secret_region != null ? can(regex("us-east-1|us-east-2|us-west-1|us-west-2|eu-west-1|eu-west-2", var.replica_secret_region)) : true)
      )
    error_message = "Allowed Secret Manager regions are us-east-1|us-east-2|us-west-1|us-west-2|eu-west-1|eu-west-2 and cannot be same as primary region."
  }
}

variable "oem_agent" {
  description = "To request EM agent set default value as ture. please note OEM agent should be requested only if the DB is going to create in public subnet."
  type        = bool
  default     = false
}

variable "oem_agent_rgs_key" {
  description = "Its generic agent registration key, its not a database account or password."
  type        = string
  default     = null
  sensitive   = true
}

variable "same_region_replica_count" {
  description = "Determines number of same region read replica"
  type        = number
  default     = 1
  validation {
    condition     = (var.same_region_replica_count) <= 5
    error_message = "Maximum of 5 SameRegion ReadReplicas Allowed."
  }
}

variable "multi_az_read_replica" {
  description = "Specifies if the RDS instance is multi-AZ on same region read replica"
  type        = bool
  default     = true
}

variable "ca_cert_identifier" {
  description = "Specifies the identifier of the CA certificate for the DB instance"
  type        = string
  default     = "rds-ca-rsa2048-g1"
}

variable "timeouts" {
  description = "default timeout allowed value is set to 2 hours"
  type = set(object(
    {
      create = string
      delete = string
    }
  ))
  default = [{
    create = "2h"
    delete = "2h"
  }]
}

variable "custom_parameter_group" {
  description = "Custom Parameter Group RDS instance"
  type        = string
  default     = "NONE"
}

variable "snapshot_cron" {
    description = "Snapshot backup Cron expression"
  type    = string
  default = "cron( 00 23 ? * * * )"
}

